import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()


class InstalledApps(SettingsComponent):
    def create(self):
        red = self.parse()
        apps_list = red.find("name", value="INSTALLED_APPS").parent.value

        log.info("Adding DRF to installed apps")
        apps_list.append("'rest_framework'")

        log.info("Adding backend app to installed apps")
        apps_list.append("'backend'")

        log.info("Dumping installed apps node")
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return "backend" in red.find("name", value="INSTALLED_APPS").parent.value.to_python()


installed_apps = InstalledApps()



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()


secure_cookie_string = """\
if not DEBUG:
    SESSION_COOKIE_SECURE = True
    CSRF_COOKIE_SECURE = True
"""


class Cookies(SettingsComponent):
    def create(self):
        red = self.parse()

        log.info("Adding cookie configuration")

        red.extend(
            [
                "\n",
                "\n",
                "# Cookies",
                "\n",
                "SHARED_DOMAIN = prod_required_env('DJANGO_SHARED_DOMAIN', default=None)",
                "\n",
                "CSRF_COOKIE_DOMAIN = SHARED_DOMAIN",
                "\n",
                "CSRF_TRUSTED_ORIGINS = SHARED_DOMAIN and [f'.{SHARED_DOMAIN}']",
                "\n",
                "SESSION_COOKIE_DOMAIN = SHARED_DOMAIN",
                "\n",
                "SESSION_ENGINE = 'django.contrib.sessions.backends.signed_cookies'",
                "\n",
                "SESSION_COOKIE_AGE = 604800  # One week in seconds",
                "\n",
                secure_cookie_string,
            ]
        )

        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return red.find("name", value="SESSION_ENGINE") is not None


cookies = Cookies()



import logging

from zygoat.components import Component
from zygoat.constants import Phases
from zygoat.utils.backend import install_dependencies


from .configuration import configuration

log = logging.getLogger()


class Flake8(Component):
    def create(self):
        dependencies = [
            "flake8",
            "flake8-mock",
            "flake8-commas",
            "flake8-quotes",
            "flake8-debugger",
            "flake8-builtins",
            "flake8-deprecated",
            "flake8-comprehensions",
        ]

        log.info("Installing flake8 dev dependencies")
        install_dependencies(*dependencies, dev=True)

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)


flake8 = Flake8(sub_components=[configuration])



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()

# If this string starts with a blank line, RedBaron completely ignores it
settings_string = """def prod_required_env(key, default, method="str"):
    \"\"\"Throw an exception if PRODUCTION is true and key is not provided\"\"\"
    if PRODUCTION:
        default = environ.Env.NOTSET
    return getattr(env, method)(key, default)

"""


class Env(SettingsComponent):
    def create(self):
        red = self.parse()

        first_import_index = red.index(red.find("importnode"))

        log.info("Inserting environ import into django settings")
        red.insert(first_import_index + 1, "import environ")

        log.info("Inserting environ constructor")

        red.insert(first_import_index + 2, "env = environ.Env()")

        log.info("Inserting prod_required_env function")
        index = red.index(red.find("name", "PRODUCTION").parent)
        red.insert(index + 1, "\n")
        red.insert(index + 2, "\n")
        red.insert(index + 3, settings_string)

        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return red.find("def", "prod_required_env") is not None


env = Env()
